/*
 * MailFaces.java
 * Copyright (c) 1996-1998 John Thomas  jthomas@cruzio.com
 *      All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for non-commercial purposes
 * is hereby granted provided that this copyright notice
 * appears in all copies.
 *
 */
import java.util.*;
import java.io.*;


/** 
** This is a very simple class that the user can run initially 
** when MailFaces is installed.  It will verify the users 
** environment and setup an invocation script that is customized 
** for the environment..
** The resulting script is "mf.bat" for Windows systems and 
** is "mf.sh" for Unix systems.    
*/
public class setup  {	

private static final int TYPE_JAVA   = 0;
private static final int TYPE_JRE    = 1;
private static final int TYPE_JVIEW  = 2;
private static final int TYPE_JAVAPM = 3;
private static final int TYPE_UNKNOWN =4;
private static final int TYPE_JREW    = 5;
private static final int TYPE_JAVAW    = 6;

private static final String[] TYPES = {"java","jre","jview","javapm","????","jrew","javaw"};

private static int 		_type	= TYPE_UNKNOWN;
private static String 	_cmd	= null;
private static boolean	_debug   = false;
private static String	_javamail = null;
 

/** 
** The main method is invoked when the application is invoked from
** the command line.  We don't really need to have any instance 
** variables so we will run everything directly from Main.
** <pre>
** java setup  [-d] [ JavaMailPath ] [ javacmd ]
**     -d      Turns on debug option
**
**     javacmd        Command name for the java command that will
*                     be used to invoke MailFaces.
**                    Default is "java"  
*                     The valid names are defined by "TYPES"
**
** Example:
**     java setup  -d c:/java/lib/mail.jar  java
** </pre>
**
*/
public static void 
main (String[] argv) {
	
String 		javacmd = null;
//File 		mf		= null;
String		fn		= null;
boolean 	win 	= false;
//boolean		jre		= false;

BufferedWriter  shell	= null;
	
	_type = TYPE_UNKNOWN;
    // The user can specify the Java VM command if they want.    
	//if (argv.length >0) { 
	for (int i=0; i < argv.length; i++) {
		String arg = argv[i];
			
		System.out.println(arg);
		// Check for debug option.
		if ( arg.equals("-d") ) {
			_debug = true;
			
			// Check for debug option.

	

		} else {
			// check if one of the choices for Java VM
			for (int j=0;j<TYPES.length;j++) {
				if (TYPES[j].equalsIgnoreCase(arg) ) {
					_type = j;
					_cmd = TYPES[j];
				}
			}
			
		}	
		  
	}
	System.out.println("**** MailFaces Setup **** "+_cmd);
	if (_type == TYPE_UNKNOWN) {
	    _cmd = "java";
	    _type = TYPE_JAVA;
	    System.out.println("Default to using 'JAVA' to invoke MailFaces.");
	}
	    
	
	//Properties p = System.getProperties();
	//p.list(System.out);	
	String version = System.getProperty("java.version");
	System.out.println("java.version="+version);
	
	// check if Java level is at least 1.1
	if (version.startsWith("1.0") ) {
		recommend11(version);
		return;
	}
	
	String os = System.getProperty("os.name");
	
	System.out.println("os.name="+os);
	if (File.separator.equals("\\") ) {
		if ( os.startsWith("Windows")  ) {
			win = true;
			fn = "mf.bat";
			if (_type == TYPE_UNKNOWN)
				getType();

		} else 
			if ( os.startsWith("OS/2") ) {
				os2note();
				if (_type == TYPE_UNKNOWN)
					_type = TYPE_JAVAPM;
				win = true;
				fn = "mf.bat";	
			} else { 
			  System.out.println("I don't know what type of system this is! ");
        System.out.println("Assume Windows.");
			  win = false;
			  fn = "mf.bat";
			  if (_type == TYPE_UNKNOWN)
					_type = TYPE_JAVA;
			} 	
			  	
	} else {
		win = false;
		fn = "mf.sh";
		// For now we will assune that the Java JDK is installed
		if (_type == TYPE_UNKNOWN)
			_type = TYPE_JAVA;

	}
	
			
	try {
		if (_cmd == null) 	
			_cmd = TYPES[_type];
		shell = new BufferedWriter(new FileWriter(fn));
		writeComments(shell,win);
		writeCmd(shell,javacmd,win);
		shell.close();
		System.out.println("\nCreated MailFaces Startup file:  "+ fn);
	
	} catch (Exception e) {
		System.out.println(e);
		e.printStackTrace();
	}
	System.exit(0);
	
} // end class main


/**
** Write some comments to the start of the shell script  
*/
private static void
writeComments(BufferedWriter shell,boolean win) 
				throws IOException 		{
	String[] comments = {
		" Invoke MailFaces ",
		" ",
		" This script is created by the setup command",
		" ",
		};
	if (win) 
		shell.write("@echo off");
	else
		shell.write("#/bin/sh");
	shell.newLine();		
	for (int i=0; i<comments.length; i++) {
		if (win) 
			shell.write("rem");
		else
			shell.write("#");
		shell.write(comments[i]);
		shell.newLine();
	}
				 
}


/**
** Write the actual command to be invoked 
**   The output for Java on windows would be:
**     set CLASSPATH=.\lib\mailfaces.jar
**	   java com.jthomas.mailfaces.MailFaces %1 %2 %3 %4 %5 %6 %7 %8 %9 
**   The output for jre would be 
**     jre -cp .\lib\mailfaces.jar com.jthomas.mailfaces.MailFaces %1 %2 ...
**   The output for jview would be 
**     jview /cp:p .\lib\mailfaces.jar com.jthomas.mailfaces.MailFaces %1 %2 ...
**  
**   The output for javapm on OS/2 would be  
**     
**   The output on Unix systems would be the same as java on Windows but with
**   "/" used instead of "\" 
*/
private static void
writeCmd(BufferedWriter shell,String cmd,boolean win) 
				throws IOException 		{
	String sep = File.pathSeparator;
	String slash = File.separator;
	String mailfacesJar = "."+slash+"lib"+slash+"mailfaces.jar";
	String javamailJar = "."+slash+"lib"+slash+"mail.jar";
	String activationJar = "."+slash+"lib"+slash+"activation.jar";
	if (win) {
		if (_type == TYPE_JAVA || _type == TYPE_JAVAW ) {
			shell.write("set CLASSPATH="+mailfacesJar+sep+
			                   javamailJar+sep+
			                   activationJar);			
								
			shell.newLine();
			shell.write(_cmd);
		
		
		
		} else if (_type == TYPE_JRE || _type == TYPE_JREW)  {
			shell.write(_cmd);			
			shell.write(" -cp "+mailfacesJar+sep+
			                   javamailJar+sep+
			                   activationJar);			

		

		} else if (_type == TYPE_JVIEW ){
			shell.write("jview /cp:p  "+mailfacesJar+sep+
			                   javamailJar+sep+
			                   activationJar);		
			

		
		} else if (_type == TYPE_JAVAPM ) {
			shell.write("set CLASSPATH="+ mailfacesJar+sep+
			                   javamailJar+sep+
			                   activationJar+sep+		
						"%CLASSPATH%");
			shell.newLine();
			shell.write("javapm");		
		} 

	} else {
		// Write cmds for Unix sh
		shell.write("CLASSPATH="+ mailfacesJar+sep+
			                   javamailJar+sep+
			                   activationJar);
		shell.newLine();	                   		
		shell.write("export CLASSPATH");
		shell.newLine();
		
		shell.write(_cmd);

	

	}
	
	// Now we add the classname to the command line.
	shell.write(" com.jthomas.mailfaces.MailFaces ");
	// Now add shell operands
	if (win) {
		if (_debug) 
			shell.write("-d ");
		shell.write("%1 %2 %3 %4 %5 %6 %7 %8 %9");
	} else {
		if (_debug) 
			shell.write("-d ");
		shell.write("$1 $2 $3 $4 $5 $6 $7 $8 $9");
	}
	shell.newLine();
	
}

/**
** getType will try to determine what type of Java Virtual Machine
** is being used to run setup and then specify that for the batch file
*/
private static void
getType() 			{
	//Properties p = System.getProperties();
	String slash = File.separator;	
	String javahome = System.getProperty("java.home");
	System.out.println("java.home="+javahome);
	File bin = new File(javahome+slash+"bin");
	//System.out.println("bin="+bin.toString());

	if (! bin.exists() ) {
		_type = TYPE_JVIEW;
		return;
	}

	File java = new File(javahome+slash+"bin"+slash+"java.exe");
	//System.out.println(java.toString());
	if (java.exists() ) 
		_type = TYPE_JAVA;
	else {
		File jre = new File(javahome+slash+"bin"+slash+"jre.exe");
		//System.out.println(jre.toString());
		if (jre.exists() ) 
			_type = TYPE_JRE;
		
		else   // assume jview
			_type = TYPE_JVIEW;
	}
}

/**
** Display a message about the specified Javamailpath not existing 
** 
*/
//private static void
//badJavamail(String arg) {
//	StringBuffer sb = new StringBuffer("");
//	sb.append("The argument '"+arg+"' is not a valid MailFaces path.");
//	sb.append("\n");
//	sb.append("Valid operands are:\n");
//	sb.append("  -d              Turns on Debug option\n");
//	
//	sb.append("  mailfacesPath    Pathname for mailfaces 'mail.jar' file\n");
//	sb.append("  cmd             Override Java VM command to be issued\n");
//	sb.append("\n");
//	sb.append("For example:\n");
//	sb.append("  jre  setup -d  c:\\java\\javamail\\mail.jar \n");
//
//
//	System.out.println(sb);
//}
	
/**
** Display a message that recommends what level the user 
** should be running.  
*/
private static void
recommend11(String version) {
	StringBuffer sb = new StringBuffer("");
	sb.append("Your Java level "+version+" will not support this application.");
	sb.append("\n");
	sb.append("The recommended level is:\n");
	sb.append("  Java Runtime Environment (JRE) 1.1) or higher \n");
	sb.append("It can be obtained from:\n");
	sb.append("http://java.sun.com/j2se/1.3/jre/index.html");
	sb.append("\n");
	System.out.println(sb);

}
/**
** Display a message that recommends what an OS/2 user should do 
** should be running.  
*/
private static void
os2note() {
	StringBuffer sb = new StringBuffer("");
	sb.append("This is an OS/2 system and running setup is not needed");	 
	sb.append("\n");
	sb.append("The recommended procedure is to run:  ");
	sb.append("faceswps.cmd");
	sb.append("\n");
	sb.append("Refer to the MailFaces OS/2 install section for more information");
	sb.append("\n");
	sb.append("http://www.cruzio.com/~jthomas/mailfaces/index.html");
	sb.append("\n");
	System.out.println(sb);

}

} // end class setup